/**************************************************************************
 *                                                                        *
 *   Copyright (c) Digital Equipment  Corporation,  1991.   All  Rights   *
 *   Reserved.  Unpublished rights reserved under the copyright laws of   *
 *   the United States.                                                   *
 *                                                                        *
 *   The software  contained  on  this  media  is  proprietary  to  and   *
 *   embodies   the   confidential   technology  of  Digital  Equipment   *
 *   Corporation.  Possession, use, duplication or dissemination of the   *
 *   software  and media is authorized only pursuant to a valid written   *
 *   license from Digital Equipment Corporation.                          *
 *                                                                        *
 *   RESTRICTED RIGHTS LEGEND Use, duplication, or  disclosure  by  the   *
 *   U.S.   Government  is  subject  to  restrictions  as  set forth in   *
 *   Subparagraph  (c)(1)(ii)  of  DFARS  252.227-7013,   or   in   FAR   *
 *   52.227-19, as applicable.                                            *
 *                                                                        *
 **************************************************************************/
#ifdef SCCS
static char sccsid[]="@(#)threshold.c	1.3 Stardent 89/12/11";
#endif
/*
			Copyright (c) 1989 by
			Stardent Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stardent Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under sccs control at Stardent in:
	/sccs3.p/avs/examples/s.threshold.c
	
*/

#include <avs/flow.h>
#include <avs/avs_data.h>
#include <avs/field.h>

/******************************************************************************/

/* 
 * This is a C example to compute the threshold of a 3D scalar field of
 * floating point numbers. 
 */  

/*
 * The threshold function examines each element of a field to see
 * whether it falls within the range specified by a minimum and maximum 
 * parameter (controlled by dials).  Elements in the range are passed 
 * unchanged to the output field, elements outside the range are 
 * set to zero in the output field.
 */

/* 
 * The function AVSinit_modules is called from the main() routine supplied
 * by AVS.  In it, we call AVSmodule_from_desc with the name of our 
 * description routine.
 */

AVSinit_modules()
{
	int threshold();

	AVSmodule_from_desc(threshold);
}

/*  The routine "threshold" is the description routine.  */

threshold()
{
	int thresh_compute();	/* declare the compute function (below) */
	int in_port, out_port;	/* temporaries to hold the port numbers */

	/* Set the module name and type */
	AVSset_module_name("threshold c", MODULE_FILTER);

	/* Create an input port for the required field input */
	in_port =
		AVScreate_input_port("Input Field",
				     "field 3D uniform scalar float", REQUIRED);

	/* Create an output port for the result */
	out_port = AVScreate_output_port("Output Field",
					 "field 3D uniform scalar float");

	/* Tell AVS to allocate space for the output data based on the size */
	/* of the input data - note that this only works when the output */
	/* port has the same type as the input port */

	AVSinitialize_output(in_port, out_port);

	/* Add two floating point parameters, both unbounded.  Min has */
	/* an initial value of zero, max of 255 */

	AVSadd_float_parameter("thresh_min", 0.0, FLOAT_UNBOUND, FLOAT_UNBOUND);
	AVSadd_float_parameter("thresh_max", 255.0, FLOAT_UNBOUND, FLOAT_UNBOUND);

	/* Tell avs what subroutine to call to do the compute */
	AVSset_compute_proc(thresh_compute);
}

/*
 * thresh_compute is the compute routine.  It is called whenever AVS wants to 
 * compute new threshold results.  The arguments are: the value of the input
 * field, the new output field (doubly indirected), the minimum parameter 
 * value and the maximum parameter value.  Note the order is always inputs,
 * outputs, parameters.  The min comes before the max because in the 
 * description routine above, the min is declared before the max.
 */

thresh_compute(input, output, pmin, pmax)
AVSfield_float *input, **output;
float *pmin, *pmax;
{
    register int i, j, k;
    register float min = *pmin;
    register float max = *pmax;

    /*
     * We use a triply nested loop to traverse the field.  The macros MAXX,
     * MAXY, and MAXZ determine the maximum extent of the field in each of
     * the three dimensions.  We know this will be a 3-dimensional
     * field because of the declaration in the description routine, so
     * we don't need to check.  When we want to reference an element of the
     * field we use the I3D macro which picks an element of a 3D field.
     * Note that the first index (i) varies the fastest in memory, so we
     * make that the innermost loop.
     */

     for (k = 0; k < MAXZ(input); k++)
	     for (j = 0; j < MAXY(input); j++)
		     for (i = 0; i < MAXX(input); i++)
			    if (I3D(input, i, j, k) > max) {
				    I3D(*output, i, j, k) = 0.0;
			    } else if (I3D(input, i, j, k) < min) {
				    I3D(*output, i, j, k) = 0.0;
			    } else {
				    I3D(*output, i, j, k) = I3D(input, i, j, k);
			    }

    /* When we're done, we return 1 to indicate success */
    return(1);
}
